#    Kye - classic puzzle game
#    Copyright (C) 2005, 2006 Colin Phipps <cph@moria.org.uk>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

import pygtk
pygtk.require('2.0')
import gtk
from kye.common import xsize, ysize, findfile, KyeImageDir

class KCanvas(gtk.DrawingArea):
    tilesize = 16
    def __init__(self, responder):
	gtk.DrawingArea.__init__(self)
	self.set_size_request(self.tilesize*xsize,self.tilesize*ysize)
	self.set_events(gtk.gdk.EXPOSURE_MASK | gtk.gdk.KEY_PRESS_MASK | gtk.gdk.KEY_RELEASE_MASK | gtk.gdk.BUTTON_PRESS_MASK | gtk.gdk.BUTTON_RELEASE_MASK | gtk.gdk.POINTER_MOTION_MASK)
	self.connect("expose-event",self.expose)
	self.mouseto = responder.mouse_motion_event
	self.bpress = responder.button_press_event
	self.brelease = responder.button_release_event
	self.connect("motion_notify_event", self.mouse_motion_event)
	self.connect("button_press_event", self.button_press_event)
	self.connect("button_release_event", self.button_release_event)
        if hasattr(responder, "key_press_event"):
            self.connect("key_press_event", responder.key_press_event)
        if hasattr(responder, "key_release_event"):
            self.connect("key_release_event", responder.key_release_event)
	self.imgdir = KyeImageDir(findfile("images.tar.gz"))
	self.images = {}
	self.set_flags(gtk.CAN_FOCUS)
	self.showboard = []
	for i in xrange(xsize*ysize): self.showboard.append("blank")

    def game_redraw(self,g, iv):
	i = -1
	for y in xrange(ysize):
	    for x in xrange(xsize):
		i = i + 1
		if iv[i] == 1:
		    iv[i] = None
		    t = g.get_tile(x,y)
		    if t != self.showboard[xsize*y+x]:
		        self.showboard[xsize*y+x] = t
		        self.queue_draw_area(self.tilesize*x,self.tilesize*y,self.tilesize,self.tilesize)

    def get_image(self, imgname):
	try:
	    i = self.images[imgname]
	except KeyError:
	    d = self.imgdir.get_tile(imgname)
	    pixbuf_loader = gtk.gdk.PixbufLoader()
	    pixbuf_loader.set_size(KCanvas.tilesize,KCanvas.tilesize)
	    pixbuf_loader.write(d)
	    pixbuf_loader.close()
	    i = pixbuf_loader.get_pixbuf()
	    if i == None:
		raise KeyError,"Incomplete image for "+imgname
	    i = i.composite_color_simple(KCanvas.tilesize,KCanvas.tilesize,gtk.gdk.INTERP_BILINEAR,255,KCanvas.tilesize,0xffffffL,0xffffffL)
	    i = i.add_alpha(False,'\x00','\x00','\x00')
	    self.images[imgname] = i
	return i

    def settilesize(self,n):
	KCanvas.tilesize = n
	self.images = {}
	self.set_size_request(self.tilesize*xsize,self.tilesize*ysize)
	self.queue_draw_area(0,0,self.tilesize*xsize,self.tilesize*ysize)

    def drawcell(self,gc,i,j):
	tile = self.showboard[i+j*xsize]

	i = i*self.tilesize
	j = j*self.tilesize

	if (tile == "blank"):
	    self.window.draw_rectangle(self.style.white_gc,True,i,j,self.tilesize,self.tilesize)
	else:
	    self.window.draw_pixbuf(gc, self.get_image(tile), 0, 0, i, j, self.tilesize, self.tilesize, gtk.gdk.RGB_DITHER_NORMAL, 0, 0)

    def expose(self, area, event):
	gc = self.window.new_gc()
	gc.set_fill(gtk.gdk.SOLID)
	gc.set_function(gtk.gdk.COPY)
	x , y, width, height = event.area

	tilesize = self.tilesize
        try:
            for i in xrange(xsize):
                if (i+1)*tilesize <= x or i*tilesize > x+width:
                    continue
                for j in xrange(ysize):
                    if (j+1)*tilesize <= y or j*tilesize > y+height:
                        continue
                    self.drawcell(gc,i,j)
        except KeyError, e:
            md = gtk.MessageDialog(type=gtk.MESSAGE_ERROR,message_format="Tileset is missing image for "+str(e),buttons=gtk.BUTTONS_OK)
            md.run()
            md.destroy()
            gtk.main_quit()

    def button_press_event(self, widget, event):
        x,y,mods = event.window.get_pointer()
        x = x / KCanvas.tilesize
        y = y / KCanvas.tilesize
        self.bpress(event.button,x,y)

    def button_release_event(self, widget, event):
        x,y,mods = event.window.get_pointer()
        x = x / KCanvas.tilesize
        y = y / KCanvas.tilesize
        self.brelease(event.button,x,y)

    def mouse_motion_event(self, widget, event):
	x,y,mods = event.window.get_pointer()
	x = x / KCanvas.tilesize
	y = y / KCanvas.tilesize
        self.mouseto(x,y)
