#    Kye - classic puzzle game
#    Copyright (C) 2006 Colin Phipps <cph@moria.org.uk>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

from os import environ
from os.path import join, basename

class KyeDefaults:
    known_header = "[Known Levels]"
    
    def __init__(self):
        self.__cf = join(environ['HOME'], ".kye.config")
        self.__count = 0
        self.__known = {}
        self.__orderfiles = {}
        self.__path = {}
        try:
            s = open(self.__cf)
            while 1:
                l = s.readline().strip()
                if l == "":
                    break
                elif l == KyeDefaults.known_header:
                    while 1:
                        l = s.readline().strip()
                        if l == "":
                            break
                        a = l.split("\t")
                        path = a.pop(0)
                        n = basename(path)
                        self.__known[n] = a
                        self.__path[n] = path
                        self.__orderfiles[n] = self.__count
                        self.__count = self.__count + 1
        
        except IOError, e:
            0

    def get_known_levels(self, n):
        if self.__known.has_key(n):
            return self.__known[n]
        return []

    def add_known(self, path, l):
        # Index by just the filename
        n = basename(path)
        self.__path[n] = path
        
        # Remember that this is the most recently loaded level
        self.__orderfiles[n] = self.__count
        self.__count = self.__count + 1

        # Add this level to the known list for this file (adding the file to the dictionary if new)
        if not self.__known.has_key(n):
            self.__known[n] = []
        if l not in self.__known[n]:
            self.__known[n].append(l)

    def get_recent(self):
        a = self.__known.keys()
        a.sort(lambda x,y: cmp(self.__orderfiles[x], self.__orderfiles[y]))
        return [self.__path[n] for n in a[-5:]]

    def save(self):
        try:
            s = open(self.__cf, "w")
            s.write(KyeDefaults.known_header+"\n")
            a = self.__known.keys()
            a.sort(lambda x,y: cmp(self.__orderfiles[x], self.__orderfiles[y]))
            for k in a:
                s.write("\t".join([self.__path[k],"\t".join(self.__known[k])])+"\n")
            s.write("\n")
        except IOError, e:
            0
