# -*- coding: UTF-8 -*-
#    Kye - classic puzzle game
#    Copyright (C) 2005 Colin Phipps <cph@moria.org.uk>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

import pygtk
pygtk.require('2.0')
import gtk
from kye.canvas import KCanvas
from kye.palette import KPalette
from kye.dialogs import KyeHelpDialog, KyeAboutDialog, llabel, getopendialog, kyeffilter
from os.path import exists
import types

ui_string = """<ui>
 <menubar name='KyeEditMenuBar'>
  <menu action='FileMenu'>
   <menuitem action='Open' />
   <menuitem action='Save' />
   <menuitem action='SaveAs' />
   <separator />
   <menuitem action='Quit' />
  </menu>
  <menu action='EditMenu'>
   <menuitem action='Undo' />
   <separator />
   <menuitem action='Auto-round walls' />
  </menu>
  <menu action='LevelMenu'>
   <menuitem action='New Level' />
   <menuitem action='Edit Level Text' />
   <menuitem action='Delete Level' />
   <separator />
  </menu>
  <menu action='ViewMenu'>
   <menuitem action='Tiny' />
   <menuitem action='Small' />
   <menuitem action='Medium' />
   <menuitem action='Large' />
  </menu>
  <menu action='HelpMenu'>
   <menuitem action='The Game' />
   <menuitem action='About Kye' />
  </menu>
 </menubar>
</ui>"""

class KEditFrame(gtk.Window):

    def delete_event(self, widget, event, data=None):
        # If you return FALSE in the "delete_event" signal handler,
        # GTK will emit the "destroy" signal. Returning TRUE means
        # you don't want the window to be destroyed.
        # This is useful for popping up 'are you sure you want to quit?'
        # type dialogs.
        if self.levelcall("modified"):
            md = gtk.MessageDialog(self,type=gtk.MESSAGE_QUESTION, buttons=gtk.BUTTONS_YES_NO, message_format="Quit without saving?")
            r = md.run()
            md.destroy()
            if r == gtk.RESPONSE_NO:
                return True
        self.__lose_ok = True
        return False

    # Another callback
    def destroy(self, widget, data=None):
        if not self.__lose_ok and self.levelcall("modified"):
            self.dosave("/var/tmp/abort.kye")
        gtk.main_quit()

    def quit(self,a):
        if self.delete_event(None,a) == True: return
        gtk.main_quit()

    def build_ui(self):
        """Construct UIManager and ActionGroup with bindings to functions here to handle menu actions"""
	ag = gtk.ActionGroup('WindowActions')

        actions = [
		("FileMenu", None, "_File"),
		("Open", gtk.STOCK_OPEN, "_Open…", "<control>O", "Open a set of levels", self.opendialog),
		("Save", gtk.STOCK_SAVE, "_Save","<control>S", "Save these levels", self.menusave),
		("SaveAs", gtk.STOCK_SAVE_AS, "Save _As…","<control>A", "Save these levels to a new filename", self.menusaveas),
		("Quit", gtk.STOCK_QUIT, "_Quit", "<control>Q", "Quit the game", self.quit),
		("EditMenu", None, "_Edit"),
		("Undo", gtk.STOCK_UNDO, "_Undo","<control>Z", "Undo last edit", self.undo),
		("LevelMenu", None, "_Level"),
		("New Level", None, "_New Level", "<control>N", "Add a new level to this set of levels", self.newlevel),
		("Edit Level Text", None, "_Edit Level Text…", "<control>E", "Edit the level name, hint and exit message for this level", self.textdialog),
		("Delete Level", None, "_Delete Level", "<control>D", "Drop this level from this level set", self.droplevel),
		("ViewMenu", None, "_View"),
		("HelpMenu", None, "_Help"),
		("The Game", gtk.STOCK_HELP, "_The Game","<control>T", "Help for playing the game", self.helpdialog),
		("About Kye", gtk.STOCK_ABOUT, "About _Kye", "<control>K", "About Python Kye", self.aboutdialog),
        ]
        ag.add_actions(actions)

        toggle_actions = [
            ("Auto-round walls", None, "Auto-_round walls", "<control>R", "Toggle whether wall corners are rounded off automatically by the editor", self.autoroundclick, True)
        ]
        ag.add_toggle_actions(toggle_actions)

        # View menu is a series of radio options
        radio_actions = [
		("Tiny", None, "_Tiny", "<control>1", "Set window size", 8),
		("Small", None, "_Small", "<control>2", "Set window size", 16),
		("Medium", None, "_Medium", "<control>3", "Set window size", 24),
		("Large", None, "_Large", "<control>4", "Set window size", 32)
        ]
        ag.add_radio_actions(radio_actions, 16, self.settilesize)

        # And now create the UI object and add accel group to the window
        self.ui = gtk.UIManager()
        self.ui.insert_action_group(ag, 0)
        self.ui.add_ui_from_string(ui_string)
        self.add_accel_group(self.ui.get_accel_group())

    def __init__(self, opencall = None, savecall = None, levelcall = None, palsource = None):
	if gtk.pygtk_version[0] < 2 or gtk.pygtk_version[1] < 2:
	    raise EnvironmentError, "Needs at least pygtk-2.2.0"
        # create a new window
        gtk.Window.__init__(self,gtk.WINDOW_TOPLEVEL)
    
        # When the window is given the "delete_event" signal (this is given
        # by the window manager, usually by the "close" option, or on the
        # titlebar), we ask it to call the delete_event () function
        # as defined above. The data passed to the callback
        # function is NULL and is ignored in the callback function.
        self.connect("delete_event", self.delete_event)
    
        # Here we connect the "destroy" event to a signal handler.  
        # This event occurs when we call gtk_widget_destroy() on the window,
        # or if we return FALSE in the "delete_event" callback.
        self.connect("destroy", self.destroy)
    
        # Sets the border width of the window.
        self.set_title("Kye Level Editor")
	self.set_resizable(False)
    
        # Create edit tool
        self.palette = KPalette(palsource)
        self.palette.settilesize(16)
      
        # Creates a new canvas, directing mouse events to the palette
        self.canvas = KCanvas(self.palette)

        # Now palette can set up
        self.palette.setup(self.canvas.get_image)
    
	# Main vbox
	self.main_vbox = gtk.VBox(False)
	self.main_vbox.set_border_width(1)

	# Build menu
        self.levels_ui_mid = self.levels_ag = None
        self.build_ui()
	self.menubar = self.ui.get_widget('/KyeEditMenuBar')
	self.main_vbox.pack_start(self.menubar, False, True, 0)
	self.menubar.show()

        # This packs the button into the window (a GTK container).
        self.main_vbox.pack_start(self.canvas, padding=0)
        self.canvas.show()

        # And the palette
        self.main_vbox.pack_start(self.palette, padding=0)
        self.palette.show()

	# Add and display the vbox
	self.add(self.main_vbox)
	self.main_vbox.show()

        # and the window
        self.set_icon(self.canvas.get_image("kye"))
        self.show()
	self.set_focus(self.canvas)
        self.set_menuitem_sensitive("/KyeEditMenuBar/FileMenu/Save",False)

        # More internal state setup
	self.opencall = opencall
	self.savecall = savecall
	self.levelcall = levelcall
        self.__lose_ok = False
        self.__fn = None

    # UI maintenance - dynamic menus and changing menu item sensitivity

    def setfname(self,fn):
        """Set filename for the currently edited file - used for Save, and in the menu bar"""
        self.__fn = fn
        self.set_menuitem_sensitive("/KyeEditMenuBar/FileMenu/Save",True)
        self.set_title("Kye Level Editor - %s" % fn)

    def set_menuitem_sensitive(self,mi,a):
        self.ui.get_widget(mi).set_sensitive(a)

    def hintmenuitems(self,**h):
        if h.has_key('dlevel'):
            self.set_menuitem_sensitive('/KyeEditMenuBar/LevelMenu/Delete Level',h['dlevel'])
        if h.has_key('undo'):
            self.set_menuitem_sensitive('/KyeEditMenuBar/EditMenu/Undo',h['undo'])

    def setlevels(self,l,cur):
        # Remove any existing level list
        if self.levels_ui_mid != None:
            self.ui.remove_ui(self.levels_ui_mid)
        if self.levels_ag != None: self.ui.remove_action_group(self.levels_ag)
        self.levels_ui_mid = self.levels_ag = None

        # Build extra menu items, and their actions, for level selection
        ag = gtk.ActionGroup("LevelActions")
        radio_actions = []
        uistring = """<ui><menubar name='KyeEditMenuBar'><menu action='LevelMenu'>"""
        n = 0
        for name in l:
            radio_actions.append(("Level%d" % n, None, name, None, "Edit level "+name, n))
            uistring = uistring + "<menuitem action='Level%d' />" % n
            n = n + 1
        uistring = uistring + "</menu></menubar></ui>"
        ag.add_radio_actions(radio_actions, cur, self.levelselect)
        self.levels_ag = ag
        self.ui.insert_action_group(ag, 1)
        self.levels_ui_mid = self.ui.add_ui_from_string(uistring)

    # Menu item handling (except file actions)

    def levelselect(self,a,u):
        """Change selected level"""
	self.levelcall("setlevel",n=a.get_current_value())

    def undo(self,a):
        try:
            self.levelcall("undo")
        except IndexError, e:
            self.error_message("No undo available")

    def newlevel(self,a):
        self.levelcall("newlevel")

    def droplevel(self,a):
        self.levelcall("droplevel")

    def autoroundclick(self,a):
        """Handle clicking on the auto-round menu checkbox"""
	self.levelcall("setautoround",s = a.get_active())

    # Now file actions

    def dochecks(self):
        errs = self.levelcall("check")
        if len(errs) == 0: return True
        errmsg = 'Errors were found in some of your levels:'
        for l in errs.keys():
            errmsg = errmsg + '\n' + l + ':' + '\n'.join(errs[l])
        md = gtk.MessageDialog(parent=self, type=gtk.MESSAGE_WARNING, buttons=gtk.BUTTONS_OK_CANCEL, message_format=errmsg)
        response = md.run()
        md.destroy()
        return response == gtk.RESPONSE_OK

    def opendialog(self,a):
        """Prompt for a .kye file to open, and then pass to opencall"""
	filesel = getopendialog()
        response = filesel.run()
        if response == gtk.RESPONSE_OK:
            filename = filesel.get_filename()
            filesel.destroy()
            self.opencall(filename)
        else:
            filesel.destroy()

    def menusaveas(self,a):
        """Ask where to save and then save the levels there"""
        if not self.dochecks():
            return
	filesel = gtk.FileChooserDialog("Save Kye Levels", action=gtk.FILE_CHOOSER_ACTION_SAVE, buttons=(gtk.STOCK_OK, gtk.RESPONSE_OK,gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT))
	filesel.add_filter(kyeffilter())
        try:
            filesel.set_do_overwrite_confirmation(True)
        except AttributeError, e:
            1        
	response = filesel.run()
        if response == gtk.RESPONSE_OK:
            filename = filesel.get_filename()
            filesel.destroy()
            self.savecall(filename)
	    self.setfname(filename)
        else:
            filesel.destroy()

    def menusave(self,a):
        """Save from the menu - check we do know the filename, and do checks, and finally pass through to the actual save function"""
        if self.__fn == None:
            # Should be impossible, menu item ought to be deselected
            self.error_message("No filename given - use Save As")
        else:
            if self.dochecks():
                self.savecall(self.__fn)

    def settilesize(self, ra, u):
        """Respond to view menu selections"""
        a = ra.get_current_value()
	self.canvas.settilesize(a)
        self.palette.settilesize(a)

    def helpdialog(self,a):
	hd = KyeHelpDialog(getimage=self.canvas.get_image, parent=self)
	hd.show()

    def textdialog(self,a):
	ad = gtk.Dialog(title='Set level text', parent=self, flags=gtk.DIALOG_MODAL, buttons=(gtk.STOCK_OK, gtk.RESPONSE_ACCEPT, gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT))
	ad.set_default_response(gtk.RESPONSE_ACCEPT)
	h = self.levelcall("getmessages")
	table = gtk.Table(3,2)
	table.attach(llabel("Level name"),0,1,0,1)
	table.attach(llabel("Hint text"),0,1,1,2)
	table.attach(llabel("End of level text"),0,1,2,3)
	e = {}
	f = { 'name' : 20, 'hint' : 35, 'exitmsg' : 50 }
	for t in f.keys():
	    e[t] = gtk.Entry(max=f[t])
	    e[t].set_width_chars(f[t])
	    e[t].set_text(h[t])
	    e[t].set_activates_default(True)
	table.attach(e['name'],1,2,0,1)
	table.attach(e['hint'],1,2,1,2)
	table.attach(e['exitmsg'],1,2,2,3)
	table.show_all()
	ad.vbox.pack_start(table, True,True,0)
	r = ad.run()
	if r == gtk.RESPONSE_ACCEPT:
	    for t in e.keys():
		h[t] = e[t].get_text()
	    self.levelcall("setmessages",**h)
	ad.destroy()

    def aboutdialog(self,a):
	ad = KyeAboutDialog(kimg = self.canvas.get_image("kye"))
        ad.run()
        ad.destroy()

    def error_message(self,message):
	md = gtk.MessageDialog(parent=self,type=gtk.MESSAGE_ERROR,message_format=message,buttons=gtk.BUTTONS_OK)
	md.run()
	md.destroy()

    def main(self):
        # All PyGTK applications must have a gtk.main(). Control ends here
        # and waits for an event to occur (like a key press or mouse event).
        gtk.main()

