#    Kye - classic puzzle game
#    Copyright (C) 2005 Colin Phipps <cph@moria.org.uk>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

from kye.objects import *
from string import upper
from common import xsize, ysize

class KGame:
    xsize = 30
    ysize = 20

    cell_lookup = {
	'K' : (Kye,),
	'1' : (Wall, 1),
	'2' : (Wall, 2),
	'3' : (Wall, 3),
	'4' : (Wall, 4),
	'5' : (Wall, 5),
	'6' : (Wall, 6),
	'7' : (Wall, 7),
	'8' : (Wall, 8),
	'9' : (Wall, 9),
	'b' : (Block, 0, False),
	'B' : (Block, 0, True),
	'a' : (Block, 1, False),
	'c' : (Block,-1, False),
	'e' : (Edible,),
	'*' : (Diamond,),
	'D' : (Sentry, 0, 1),
	'U' : (Sentry, 0,-1),
	'L' : (Sentry,-1, 0),
	'R' : (Sentry, 1, 0),
	'[' : (Monster, 2),
	'E' : (Monster, 0),
	'T' : (Monster, 1),
	'~' : (Monster, 3),
	'C' : (Monster, 4),
	's' : (Magnet, 0, 1),
	'S' : (Magnet, 1, 0),
	'u' : (Slider, 0,-1,False),
	'd' : (Slider, 0, 1,False),
	'l' : (Slider,-1, 0,False),
	'r' : (Slider, 1, 0,False),
	'^' : (Slider, 0,-1, True),
	'v' : (Slider, 0, 1, True),
	'<' : (Slider,-1, 0, True),
	'>' : (Slider, 1, 0, True),
	'H' : (BlackHole,),
	'}' : (Block, 0, False, 3),
	'|' : (Block, 0, False, 4),
	'{' : (Block, 0, False, 5),
	'z' : (Block, 0, False, 6),
	'y' : (Block, 0, False, 7),
	'x' : (Block, 0, False, 8),
	'w' : (Block, 0, False, 9),
	'h' : (OneWay, 0, 1),
	'i' : (OneWay, 0,-1),
	'f' : (OneWay, 1, 0),
	'g' : (OneWay,-1, 0),
	'A' : (Shooter,False),
	'F' : (Shooter,True),
	}

    def __init__(self, f, lev, movesource, rng):
	levels = f.readline()
	self.levelnum = 0
	while 1:
	    l = upper(f.readline())
	    self.levelnum = self.levelnum+1
	    l = l.strip()
	    if l == "": break
	    if (l == lev or "" == lev): break
	    for i in xrange(22):
	        l = f.readline()

	if (l != lev and lev != ""):
	    raise KeyError, "level "+lev+" not found"

	self.thislev = l
	self.hint = f.readline()
	self.exitmsg = f.readline()
	board = [];
	self.invalidate = []
	self.magnet_count = []

	for i in xrange(KGame.xsize*KGame.ysize):
	    board.append(None)
	    self.invalidate.append(1)
	    self.magnet_count.append(0)

	self.board = board
	self.loc = {}
	self.thinkers = []
	self.diamonds = 0

	for y in xrange(KGame.ysize):
	    l = f.readline()
	    for x in xrange(KGame.xsize):
                edge = (x == 0 or y == 0 or x == KGame.xsize-1 or y == KGame.ysize-1)
                c = l[x]

                # Blank means empty cell - but override at the edge
                if c == ' ' and edge:
                    c = '5'
		if c == ' ':
		    continue
		try:
		    ctype = KGame.cell_lookup[c]

                    # edge cells must be wall cells
                    if ctype[0] != Wall and edge:
                        ctype = (Wall, 5)
                    
                    # Execute constructor for this object type, and add to the grid
		    cc = apply(ctype[0],ctype[1:])
		    self.add_at(x,y,cc)

                    # Objects where location matters
		    if isinstance(cc,Shooter):
			cc.setang((x+y)%4)
		    if isinstance(cc,Kye):
			self.kyestart = (x,y)
			self.thekye = cc
                    
		except KeyError, e:
		    print "Unable to load %s at (%d,%d)" % (c,x,y)

	self.nextlevel = f.readline().strip()
	if self.nextlevel == "\x1a": self.nextlevel=""
	f.close()

	self.animate_frame = 0
	self.running = 1
	self.tics = 0
	self.random = rng
	self.ms = movesource

    def nextrand(self,n):
	return self.random.randint(0,n-1)

    def get_at(self,i,j):
	return self.board[KGame.xsize*j + i]

    def get_atB(self,i,j):
	if i < 0 or i >= KGame.xsize or j < 0 or j >= KGame.ysize: return Wall(5)
	return self.get_at(i,j)

    def get_tile(self,i,j):
	c = self.get_at(i,j)
	if (c == None): return "blank"
	return c.image(self.animate_frame)

    def get_location(self,obj):
	return self.loc[obj]

    def magnet_range(self,x,y,d):
	pos = KGame.xsize*y + x
	max = KGame.xsize*KGame.ysize
	if pos >= 2: self.magnet_count[pos - 2] = self.magnet_count[pos - 2] + d
	if pos >= 1: self.magnet_count[pos - 1] = self.magnet_count[pos - 1] + d
	if pos + 1 < max: self.magnet_count[pos + 1] = self.magnet_count[pos + 1] + d
	if pos + 2 < max: self.magnet_count[pos + 2] = self.magnet_count[pos + 2] + d
	if pos >= 2*KGame.xsize: self.magnet_count[pos - 2*KGame.xsize] = self.magnet_count[pos - 2*KGame.xsize] + d
	if pos >= 1*KGame.xsize: self.magnet_count[pos - 1*KGame.xsize] = self.magnet_count[pos - 1*KGame.xsize] + d
	if pos + 2*KGame.xsize < max: self.magnet_count[pos + 2*KGame.xsize] = self.magnet_count[pos + 2*KGame.xsize] + d
	if pos + 1*KGame.xsize < max: self.magnet_count[pos + 1*KGame.xsize] = self.magnet_count[pos + 1*KGame.xsize] + d

    def add_at(self,x,y,obj):
	pos = KGame.xsize*y + x
	self.board[pos] = obj
	self.invalidate[pos] = 1
	self.loc[obj] = (x,y)
	if isinstance(obj,Kye): self.kye = obj
	f = obj.freq()
	if f > 0: self.thinkers.append((f,obj))
	if isinstance(obj,Diamond): self.diamonds = self.diamonds+1
	if isinstance(obj,Magnet): self.magnet_range(x,y,1)

    def remove_at(self,x,y):
	pos = KGame.xsize*y + x
	obj = self.board[pos]
	self.board[pos] = None
	self.invalidate[pos] = 1
	f = obj.freq()
	if f > 0: self.thinkers.remove((f,obj))
	if isinstance(obj,Kye): self.kye = None
	del self.loc[obj]
	if isinstance(obj,Diamond): self.diamonds = self.diamonds-1
	if isinstance(obj,Magnet): self.magnet_range(x,y,-1)

    def move_object(self,x,y,tx,ty):
	pos_f = KGame.xsize*y + x
	pos_t = KGame.xsize*ty + tx
	obj = self.board[pos_f]
	self.board[pos_f] = None
	self.board[pos_t] = obj
	self.invalidate[pos_f] = 1
	self.invalidate[pos_t] = 1
	self.loc[obj] = (tx,ty)
	if isinstance(obj,Magnet):
	    self.magnet_range(x,y,-1)
	    self.magnet_range(tx,ty,1)

    def invalidate_me(self,o):
	x,y = self.get_location(o)
	self.invalidate[KGame.xsize*y + x] = 1

    def push_object(self,x,y,dx,dy):
	tx,ty = x+dx,y+dy
	obj = self.get_at(x,y)
	if not isinstance(obj,Thinker): return False
	
	# To push diagonally, need the two side squares clear
	if dx != 0 and dy != 0:
	    if self.get_atB(x,ty) != None or self.get_atB(tx,y) != None:
		return False

	# Target square
	t = self.get_atB(tx,ty)
	if t == None:
	    self.move_object(x,y,tx,ty)
	    return True
	elif isinstance(t,BlackHole):
	    if t.swallow(self):
		self.remove_at(x,y)
		return True
	return False

    def dokye(self,k):
	x,y = self.get_location(k)
	m = self.ms.get_move()
	if m == None: return
	if m[0] == 'abs':
	    if m[1] < x: dx = -1
	    elif m[1] > x: dx = 1
	    else: dx = 0
	    if m[2] < y: dy = -1
	    elif m[2] > y: dy = 1
	    else: dy = 0
	else:
	    dx,dy = m[1:]
	if dx != 0 and dy != 0:
	    # Diagonal move. If either square either side of the diagnoal is occopied, we cannot move diagonally - but we could still move into the other square beside the diagonal
	    xt,yt = self.get_atB(x+dx,y),self.get_atB(x,y+dy)
	    if m[0] == 'abs':
		# Fail if both blocked
		if xt != None and yt != None: return

		# Otherwise, step which way we can
		if xt != None: dx = 0
		if yt != None: dy = 0
	    else:
		# Fail if even one blocked if he definitely wanted diagonal
		if xt != None or yt != None: return
	t = self.get_atB(x+dx,y+dy)
	if isinstance(t,Edible):
	    self.remove_at(x+dx,y+dy)
	    t = None
	if isinstance(t,BlackHole):
	    if t.swallow(self, animate = False):
	        self.kill_kye(k)
	else:
	    nu = None
	    if isinstance(t,OneWay) and t.allow_move(dx,dy):
		self.remove_at(x+dx,y+dy)
		nu,t = t,None
	    if t == None or self.push_object(x+dx,y+dy,dx,dy):
		self.move_object(x,y,x+dx,y+dy)
		if k.under != None:
		    self.add_at(x,y,k.under)
		k.under = nu

    def find_kye(self):
	return self.get_location(self.kye)

    def kill_kye(self,k):
	x,y=self.get_location(k)
	k.lives = k.lives - 1
	ghost = KyeGhost(k)
	self.remove_at(x,y)
	self.add_at(x,y,ghost)

    def respawn_kye(self,k):
	# deja vu here - G_CheckSpot clone
	for r in xrange(40):
	    # Make some # of tries proportional to the radius
	    for t in xrange(4*r+1):
		if r == 0:
		    x,y = self.kyestart
		else:
		    x = self.kyestart[0] + self.random.randint(-r,r)
		    y = self.kyestart[1] + self.random.randint(-r,r)
		if self.get_atB(x,y) == None:
		    self.add_at(x,y,k)
		    return k

    def check_monsters(self):
	x,y = self.get_location(self.kye)
	if isinstance(self.get_atB(x+1,y),Monster) or isinstance(self.get_atB(x-1,y),Monster) or isinstance(self.get_atB(x,y+1),Monster) or isinstance(self.get_atB(x,y-1),Monster):
	    self.kill_kye(self.kye)
	    return True
	return False

    def dotick(self):
	if not self.running: return

	tics = self.tics = self.tics + 1

	if self.kye != None and not self.check_monsters():
	    self.dokye(self.kye)
	    if self.kye: self.check_monsters()

	# Shallow copy the thinkers list, because we must iterate over it while modifying it
	current_thinkers = self.thinkers[:]
	for f,t in current_thinkers:
	    if tics % f == 0:
		try:
		    x,y = self.get_location(t)
		except KeyError, e: # Object was deleted during this tick
		    continue
		if t.think(self,x,y):
		    self.invalidate[x+y*KGame.xsize]=1

	if self.tics % 3 == 0: self.animate_frame = self.animate_frame + 1
